@@ -27,8 +27,14 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+'common interface for tools to report issues'
+
+from __future__ import absolute_import, print_function, unicode_literals
+
 
 class Issue:
+    '''An issue has a file, location, and message. OPtionally also
+    categories ans severity'''
 
     def __init__(self, path, message, checker_id,
                  line_number_start=None, line_number_end=None,
