@@ -27,19 +27,16 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import print_function
+'pyflakes plugin'
+
+from __future__ import absolute_import, print_function, unicode_literals
 from unilint.python_source_plugin import AbstractPythonPlugin
 from unilint.unilint_plugin import UnilintPluginInitException
 from unilint.issue import Issue
 
-try:
-    import pyflakes
-    PYFLAKES_VERSION = pyflakes.__version__
-except ImportError as impe:
-    PYFLAKES_VERSION = None
-
 
 class PyflakesPlugin(AbstractPythonPlugin):
+    'pyflakes plugin'
 
     def __init__(self, shell_function):
         super(PyflakesPlugin, self).__init__(shell_function)
@@ -49,18 +46,19 @@ class PyflakesPlugin(AbstractPythonPlugin):
         return 'pyflakes'
 
     def get_meta_information(self):
-        if not PYFLAKES_VERSION:
+        cmd = "which pyflakes"
+        value, output, message = self.shell_cmd(cmd,
+                                                shell=True,
+                                                us_env=True,
+                                                ignore_returncodes=[127])
+        if value == 0:
+            return "pyflakes detected"
+        else:
             raise UnilintPluginInitException(
                 "ERROR Can't find pyflakes, install via apt or pip\n%s" % impe)
-        return "pyflakes: %s" % PYFLAKES_VERSION
-
-    @classmethod
-    def is_enabled_by_default(cls):
-        # pyflakes returns a subset of pylint, is only useful for
-        # quicker checks
-        return False
 
     def check_resource(self, options, path, type_categories):
+        'runs the tool and processes the output'
         if not 'python-src' in type_categories and \
                 not 'script' in type_categories:
             return None
@@ -75,9 +73,10 @@ class PyflakesPlugin(AbstractPythonPlugin):
             print(output)
             return None
         else:
-            return self.parse(output)
+            return (self.parse(output), None)
 
     def parse(self, output):
+        'creates an issue from an output line'
         issues = []
         for line in output.splitlines():
             if line.strip() == '':
