@@ -27,7 +27,9 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import print_function
+'pylint plugin'
+
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import re
 import logging
@@ -39,9 +41,11 @@ from unilint.issue import Issue
 
 
 NEWLINE_MATCHER = re.compile('^.+:[0-9]+: \[')
+INDICATOR_MATCHER = re.compile('^\s*\^+\s*\^*$')
 
 
 class PylintPlugin(AbstractPythonPlugin):
+    'pylint plugin'
 
     def __init__(self, shell_function):
         super(PylintPlugin, self).__init__(shell_function)
@@ -62,8 +66,10 @@ class PylintPlugin(AbstractPythonPlugin):
             "ERROR Cannot find pylint, install via apt or pip\n%s" % message)
 
     def check_resource(self, options, path, type_categories):
+        'runs the tool and processes the output'
         if not 'python-src' in type_categories and \
-                not 'script' in type_categories:
+                not 'script' in type_categories and \
+                not 'python-package' in type_categories:
             return None
 
         levels = []
@@ -83,7 +89,7 @@ class PylintPlugin(AbstractPythonPlugin):
             suppress_options.append('E0611')
         if options.level > LEVELS[LEVEL_ALL]:
             # 'Locally disabling ...'
-            suppress_options.append('I0011')
+            suppress_options.extend(['I0011', 'R0801'])
 
         suppress_options_string = ''
         if suppress_options:
@@ -100,27 +106,35 @@ class PylintPlugin(AbstractPythonPlugin):
                 lvl_cmd,
                 suppress_options_string))
         log = logging.getLogger(__name__)
-        log.info(cmd)
+        log.debug(cmd)
 
         _, output, _ = self.shell_cmd(
             cmd,
             shell=True,
             us_env=True,
-            cwd=os.path.dirname(os.path.abspath(path)),
             ignore_returncodes=range(50))
         output += '\n'
-        basepath = os.path.dirname(path)
         if options.raw:
             print(output)
-            return None
-        return self.parse(output, basepath)
+            return []
+        result = self.parse(output)
+        log.debug('%s issues found' % len(result))
+        ignores = None
+        if 'python-package' in type_categories:
+            ignores = ['^%s.*py$' % path]
+        return (result, ignores)
 
-    def parse(self, output, basepath):
+    def parse(self, output):
+        'creates an issue from an output line'
         # Example output:
-        #         dotcode_tf.py:39: [C0111, comp1] Missing docstring
-        #         dotcode_tf.py:47: [C0322, com1.__init__] Operator empty
-        #         self.listen_duration=1
-        #                             ^
+        # foo/bar/dotcode_tf.py:39: [C0111, comp1] Missing docstring
+        # foo/bar/dotcode_tf.py:47: [C0322, com1.__init__] Operator empty
+        # self.listen_duration=1
+        #                     ^
+        # test/integration/pep8_test.py:1: [R0801] Similar lines in 22 files
+        # ==integration.all_plugins_test:16
+        # ==integration.all_plugins_test:25
+        # ==integration.all_plugins_test:34
         issues = []
 
         lines = []
@@ -129,23 +143,30 @@ class PylintPlugin(AbstractPythonPlugin):
             if line.strip() == '':
                 continue
             if NEWLINE_MATCHER.match(line) or len(lines) == 0:
-                lines.append(line)
+                lines.append([line, None])
             else:
-                lines[-1] += "\n%s" % line
+                lines[-1][0] += "\n%s" % line
+                if INDICATOR_MATCHER.match(line):
+                    lines[-1][1] = 'indicator'
 
-        for line in lines:
+        for (line, flag) in lines:
             issue_lines = line.splitlines()
-            message_line = issue_lines[0]
-            if len(issue_lines) > 1:
-                # source = issue_lines[1]
+            multiline_message = False
+            position = None
+            if len(issue_lines) == 1:
+                message_line = issue_lines[0]
+            elif len(issue_lines) == 3 and flag == 'indicator':
+                message_line = issue_lines[0]
                 position = len(issue_lines[2]) - 1
             else:
-                # source = None
-                position = None
-
+                # multifile error
+                message_line = issue_lines[0]
+                multiline_message = True
             filename, line_number, rest = message_line.split(':', 2)
 
             category, message = rest.lstrip('[ ').split(']', 1)
+            if multiline_message:
+                message += ('\n%s' % '\n'.join(issue_lines[1:]))
             if ' ' in category:
                 category = category.split()[0]
 
@@ -161,7 +182,7 @@ class PylintPlugin(AbstractPythonPlugin):
             if category.startswith('W'):
                 severity = "warning"
 
-            issue = Issue(path=os.path.join(basepath, filename),
+            issue = Issue(path=filename,
                           message=message.strip(),
                           checker_id=self.get_id(),
                           line_number_start=line_number,
