@@ -27,6 +27,9 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+'simple formatting of issues'
+
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import sys
 from unilint.unilint_main import register_formatter
@@ -36,8 +39,10 @@ from unilint.custom_format import print_formatted
 
 
 def register_standard_formatters():
+    '''3 simple formatting functions'''
 
     def print_brief(unused_options, issue_list, basepath):
+        'prints relpath, location, message'
         if issue_list is None:
             return
         for issue in issue_list:
@@ -54,9 +59,11 @@ def register_standard_formatters():
             sys.stdout.write("%s   %s\n" % (location, issue.message))
 
     def print_full(unused_options, issue_list, basepath):
+        'prints all issue information'
         print_formatted('[$C\t$S\t$T]$F:$L:$P:\t$M', issue_list, basepath)
 
     def print_short(unused_options, issue_list, basepath):
+        'prints filename, line and message'
         print_formatted('$f:$L: $M', issue_list, basepath)
 
     register_formatter('brief', print_brief)
