@@ -29,10 +29,11 @@
 
 "contains main function and helpers"
 
-from __future__ import print_function
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import sys
 import logging
+import re
 from argparse import ArgumentParser
 
 from unilint.__version__ import VERSION
@@ -140,49 +141,66 @@ def resolve_plugins(selected_plugins_string,
 
 
 def index_different(id1, id2):
-    '''quick check returning true only if both values as ints are equal'''
+    '''quick check returning true only if both values as ints positive
+    and different'''
     if not id1 or not id2:
         return False
+    if int(id1) < 0 or int(id2) < 0:
+        return False
     if int(id1) != int(id2):
         return True
     return False
 
 
+def select_best_dupe(issue1, issue2):
+    '''Given two issues identified as duplicate, return the one with
+    higher severity, and then the one having more location
+    information'''
+    if issue2.severity < issue1.severity:
+        return issue2
+    elif (issue2.severity == issue1.severity and
+          (not issue1.line_position or int(issue1.line_position) < 0) and
+          (issue2.line_position and int(issue2.line_position) >= 0)):
+        return issue2
+    return issue1
+
+
 def remove_duplicates(issues):
     """
     looks for lines in list of issues having the same line number and
     indicating the same problem
+
     :param issues: list of Issue
     """
     if not issues:
         return issues
-    issues2 = order_issues(issues)
+    issues_ord = order_issues(issues)
     result = []
     sameline_issues = []
     lastline = -1
-    lastcolumn = -1
 
-    for issue in issues2:
+    for issue in issues_ord:
         is_likely_duplicate = True
-        if (index_different(issue.line_number_start, lastline)):
+        if not sameline_issues:
             is_likely_duplicate = False
-        elif (index_different(issue.line_position, lastcolumn)):
-            is_likely_duplicate = False
-
-        if not is_likely_duplicate:
+        elif (index_different(issue.line_number_start, lastline)):
             sameline_issues = []
             lastline = int(issue.line_number_start or -1)
-            lastcolumn = int(issue.line_position or -1)
-        else:
+            is_likely_duplicate = False
+
+        if is_likely_duplicate:
             # check whether new issue is a duplicate
             is_duplicate = False
             for oldissue in sameline_issues:
+                if index_different(issue.line_position,
+                                   oldissue.line_position):
+                    continue
                 # for now, will rely on manually crafted duplicate
                 # detection for similar error in different code analysers
                 for keyword_vec in [('too long', ),
                                     ('indentation contains', ),
                                     ('variable ', ),
-                                    ('not used', 'unused'),
+                                    ('not used', 'unused', 'Unused'),
                                     ('one statement', 'multiple statements')]:
                     for keyword_match_old in keyword_vec:
                         if keyword_match_old in oldissue.message:
@@ -193,6 +211,7 @@ def remove_duplicates(issues):
                     if is_duplicate:
                         break
             if is_duplicate:
+                result[-1] = select_best_dupe(result[-1], issue)
                 continue
         sameline_issues.append(issue)
         result.append(issue)
@@ -202,6 +221,7 @@ def remove_duplicates(issues):
 def print_issues(args, issues, path):
     """
     print to stdout
+
     :param args: all CLI arguments
     :param issues: list of Issue to print
     :param path: basepath to which to append issue relative path
@@ -215,18 +235,22 @@ def print_issues(args, issues, path):
 def order_issues(issues):
     """
     sorts Issues by line number and line position
+
     :param issues: list of Issue
     """
     def keyfun(iss):
-        return "%s%sz%s" % (iss.path,
-                            str(iss.line_number_start).rjust(5, '0'),
-                            str(iss.line_position).rjust(3, '0'))
+        '''helper to sort issues'''
+        lin = str(iss.line_number_start).rjust(5, str('0'))
+        pon = str(iss.line_position).rjust(3, str('0'))
+        return "%s%sz%s" % (os.path.abspath(iss.path), lin, pon)
+
     if not issues:
         return issues
     return sorted(issues, key=keyfun)
 
 
 class IgnorableFolderException(Exception):
+    '''Helper to break out of multiple loops'''
     pass
 
 
@@ -239,11 +263,13 @@ def check_path(path_to_check,
     """
     runs all selected plugins on the given files in order,
     optionally printing issue messages as soon as available
+
     :param path_to_check: path to file/folder in which to run checks on files
     :param basepath: absolute path that user gave in cli
     :param args: all CLI argument
     :param subdirs: subdirectories
     :param files: the files to check
+    :param force: if True, also scans tests, docs and generated folders
     """
     categorized_resources = {}
     for plugin in args.selected_plugins.values():
@@ -267,23 +293,37 @@ def check_path(path_to_check,
                         raise IgnorableFolderException()
         extend_maybe(categorized_resources, new_categories)
     issues = []
+    plugin_ignores = {pid: [] for pid in args.selected_plugins.values()}
     for resource, categories in sorted(categorized_resources.items()):
         resource_issues = []
         if 'hidden' in categories:
             continue
+        if 'broken' in categories:
+            continue
         if 'backup' in categories:
             continue
         log = logging.getLogger(__name__)
         log.debug("processing: %s as %s" % (resource, categories))
         for plugin in args.selected_plugins.values():
+            ignore_plugin = False
+            for ignore_pattern in plugin_ignores[plugin]:
+                if re.match(ignore_pattern, resource):
+                    ignore_plugin = True
+                    break
+            if ignore_plugin:
+                continue
             try:
                 result = plugin.check_resource(args,
                                                resource,
                                                categories)
-                extend_maybe(resource_issues, result)
-                if result is not None and not args.ordered and not args.raw:
-                    # get the output to the user as quickly as possible
-                    print_issues(args, result, basepath)
+                if result:
+                    (new_issues, ignores) = result
+                    extend_maybe(resource_issues, new_issues)
+                    if ignores:
+                        plugin_ignores[plugin].extend(ignores)
+                    if not args.ordered and not args.raw:
+                        # get the output to the user as quickly as possible
+                        print_issues(args, new_issues, basepath)
             except KeyboardInterrupt as kbi:
                 raise UnilintException(
                     'Interrupted while running %s on %s: %s' %
@@ -474,6 +514,7 @@ def get_unilint_parser(description=None):
 def evaluate_options(args):
     """
     check command line options are valid
+
     :returns: args, path
     """
     if len(args.path) == 0:
@@ -497,7 +538,7 @@ def evaluate_options(args):
     logoutput = logging.StreamHandler()
     logoutput.propagate = False
     logoutput.setLevel(logging.NOTSET)
-    log = logging.getLogger(__package__)
+    log = logging.getLogger(__name__.rpartition('.')[0])
     log.handlers = []
     log.propagate = False
     log.addHandler(logoutput)
@@ -507,9 +548,10 @@ def evaluate_options(args):
             '[%(asctime)s-%(name)s]%(levelname)s: %(message)s')
         logoutput.setFormatter(formatter)
         log.setLevel(logging.DEBUG)
-        log.debug('Debug log enabled')
+        log.debug('Debug log enabled for %s' % log.name)
     elif args.verbose:
         log.setLevel(logging.INFO)
+        log.debug('Verbose logging enabled for %s' % log.name)
 
     return args, paths
 
